package lib; //ELEMENT_List.java:
import java.lang.String;

//2000-06-15  Knabe  Zuordnung zu Paket lib
//2000-03-17  Knabe  Creation
//Autor: Christoph Knabe

/** Generische Klasse fuer typsichere Verpackung einer java.util.List.
  * Eine ELEMENT_List kann nur mit Elementen des Typs ELEMENT gefllt
  * werden! Dadurch ist bei der Entnahme keine Typspezialisierung
  * notwendig.
  * Nur die wichtigsten Methoden von List werden durchgereicht!
  * Instanzierung: Globales Ersetzen aller "ELEMENT" durch "Typname"
*/

/*
 * @see Collection
 * @see Set
 * @see ArrayList
 * @see LinkedList
 * @see Vector
 * @see Arrays#asList(Object[])
 * @see Collections#nCopies(int, Object)
 * @see Collections#EMPTY_LIST
 * @see AbstractList
 * @see AbstractSequentialList
 * @since JDK1.2
 */

public class ELEMENT_List {


/*The wrapped List*/
private final java.util.List _wrapped;

public ELEMENT_List(){_wrapped = new java.util.ArrayList();}
public ELEMENT_List(final java.util.List i_list)throws IllegalElementType
{
  checkAll(i_list);
  _wrapped = i_list;
}

// Query Operations:
public int size(){return _wrapped.size();}
public boolean isEmpty(){return _wrapped.isEmpty();}
public boolean contains(ELEMENT o){return _wrapped.contains(o);}
public String toString(){return _wrapped.toString();}
//?Iterator iterator();
//?Object[] toArray();
//?Object[] toArray(Object a[]);

// Modification Operations:
public boolean add(ELEMENT o){return _wrapped.add(o);}
public boolean remove(ELEMENT o){return _wrapped.remove(o);}

// Bulk Modification Operations: ???zustzlich fr ELEMENT_List?
public boolean containsAll(java.util.Collection c)
{return _wrapped.containsAll(c);}
public boolean addAll(java.util.Collection c)
throws ELEMENT_List.IllegalElementType {checkAll(c); return _wrapped.addAll(c);}
public boolean addAll(int index, java.util.Collection c)
throws ELEMENT_List.IllegalElementType
{checkAll(c); return _wrapped.addAll(index,c);}
public boolean removeAll(java.util.Collection c){return _wrapped.removeAll(c);}
public boolean retainAll(java.util.Collection c){return _wrapped.retainAll(c);}
public void clear(){_wrapped.clear();}

// Comparison and hashing:
public boolean equals(Object o)
{return _wrapped.equals(((ELEMENT_List)o)._wrapped);}
public int hashCode(){return _wrapped.hashCode();}

// Positional Access Operations:
public ELEMENT get(int index){return (ELEMENT)_wrapped.get(index);}
public ELEMENT set(int index, ELEMENT element){
  return (ELEMENT)_wrapped.set(index,element);
}
public void add(int index, ELEMENT element){_wrapped.add(element);}
public ELEMENT remove(int index){return (ELEMENT)_wrapped.remove(index);}

// Search Operations:
public int indexOf(ELEMENT o){return _wrapped.indexOf(o);}
public int lastIndexOf(ELEMENT o){return _wrapped.lastIndexOf(o);}

// List Iterators:
Iterator listIterator(){return new Iterator(_wrapped.listIterator());}
Iterator listIterator(int index)
{return new Iterator(_wrapped.listIterator(index));}

// View:
ELEMENT_List subList(int fromIndex, int toIndex){
  final java.util.List subList = _wrapped.subList(fromIndex, toIndex);
  return new ELEMENT_List(subList, true); //unchecked
}

//Additions for ELEMENT_List:

/*Returns an unmodifiable view of the wrapped Object-list.
  Useful, if you want to process the contents of this list by the java.util.List
  or the java.util.Collection interface. The returned view is unmodifiable in
  order to guarantee the type-safety. Query operations on the returned list
  "read through" to the specified list, and attempts to modify the returned
  list, whether direct or via its iterator, result in an
  UnsupportedOperationException.

  @see java.lang.UnsupportedOperationException
*/
public java.util.List objectList()
{return java.util.Collections.unmodifiableList(_wrapped);}


public static class IllegalElementType extends Exception {
  public IllegalElementType(final String i_typeName){super(i_typeName);}
}

/**@throws IllegalElementType If there is an element of i_collection, which is
  not of type ELEMENT.
*/
public static void checkAll(final java.util.Collection i_collection)
throws IllegalElementType
{
  for(final java.util.Iterator i = i_collection.iterator(); i.hasNext(); ){
    final Object o = i.next();
    if(!(o instanceof ELEMENT)){
      throw new IllegalElementType(o.getClass().getName());
    }
  }
}


/**
  A type-safe iterator for an ELEMENT_List that allows the programmer to
  traverse the list in either direction and modify the list during iteration.
  Built like java.util.ListIterator and documented there.
*/
public static class Iterator {
  private java.util.ListIterator _wrapped;
  private Iterator(final java.util.ListIterator i_listIterator){
    _wrapped = i_listIterator;
  }

  // Query Operations
  public boolean hasNext(){return _wrapped.hasNext();}
  public ELEMENT next(){return (ELEMENT)_wrapped.next();}
  public boolean hasPrevious(){return _wrapped.hasPrevious();}
  public ELEMENT previous(){return (ELEMENT)_wrapped.previous();}
  public int nextIndex(){return _wrapped.nextIndex();}
  public int previousIndex(){return _wrapped.previousIndex();}

  // Modification Operations
  public void remove(){_wrapped.remove();}
  public void set(ELEMENT o){_wrapped.set(o);}
  public void add(ELEMENT o){_wrapped.add(o);}
}


public static abstract class Comparator implements java.util.Comparator {
  public final int compare(Object object1, Object object2){
    final ELEMENT element1 = (ELEMENT)object1;
    final ELEMENT element2 = (ELEMENT)object2;
    return compare(element1, element2);
  }

  /**should be implemented by implementor of this abstract class*/
  public abstract int compare(ELEMENT element1, ELEMENT element2);
}

/**Sorts the elements of this ELEMENT_List into ascending order, according to
  the natural ordering of its elements.
*/
public void sort(){java.util.Collections.sort(_wrapped);}

/**Sorts the elements of this ELEMENT_List using the given typed comparator.
  @see java.util.Comparator
*/
public void sort(final Comparator i_comparator){
  java.util.Collections.sort(_wrapped, i_comparator);
}

public void reverse(){java.util.Collections.reverse(_wrapped);}

/**unchecked private constructor to wrap a java.util.List into an ELEMENT_List*/
private ELEMENT_List(final java.util.List i_list, final boolean i_unchecked)
{ _wrapped = i_list; }


}//ELEMENT_List

/**Generischer Platzhalter. Nur fr Compilierbarkeit*/
class ELEMENT {}