package lib; //Exc.java
import java.lang.String;

//2000-06-29  Knabe  _cause gets type Exception rather than Throwable
//2000-06-19  Knabe  printStackTrace() bei Ursachenkette ohne Ausgabe
//                   von redundanten Zeilen.
//2000-06-12  Knabe  alle printStackTrace(...) ber printStackTrace(PrintWriter)
//1999-03-23  Knabe  Parametrierung wie in Java-Tutorial/i18n/format
//1999-03-12  Knabe  NEU: causingThrowable()
//1999-01-14  Knabe  erstellt

/** Fully diagnostic Java exception with exception parameters.
  This class serves for the framework user as a base class for defining
  problem-related exceptions.

  Naming convention:
    All user-defined exception classes derived from Exc should have a name
    ending in Exc. The pattern is: <PRE>
      class ErrorconditionExc extends Exc { ... }
    </PRE>

  The exception parameters are stored in an internal Object[] and can
  be substituted into the corresponding message pattern in a desired locale
  format by class MsgText.
  <br>
  See the usage examples 'Copy', 'AwtCopy', 'SwingCopy' in directory 'demo'.
  <br>
  The exception parameters are stored in a private Object[]-attribute
  of this Exc.

  @author Christoph Knabe, TFH Berlin, 1999-2000 Copyrighted
*/

public class Exc extends java.lang.Exception {


/** Constructs an Exc, giving exception and thus message parameters.
  You should create the message parameters by initializing an Object[],e.g.:
  <CODE><PRE>
  class Overflow extends lib.Exc {
    public Overflow( final int i_size, final String i_element ){
      super(new Object[]{new Integer(i_size), i_element});
    }
  }
  </PRE>
  @param i_parameters A polymorphic array of diagnostic parameters to the
    exception suitable for message text formatting by java.text.MessageFormat.
    The array may have at most 10 elements, which must correspond to the
    message format elements {0} to {9} in the message text file {@link MsgText}.
*/
public Exc(final Object[] i_parameters){
  _parameters = i_parameters;
}

/** Constructs an Exc without message parameters
*   @see #Exc(Object[])
*/
public Exc(){
  _parameters = null;
}

/**
* Returns the detail message and parameters of this Exc object.
*
* @return  the detail message followed by the parameters of this
*          <code>Exc</code>,
*          or <code>null</code> if this <code>Exc</code> does
*          have neither. This is aimed at a compact error message format.
*/
public String getMessage() {
  System.err.println("Exc.getMessage()");
  final java.lang.reflect.Field[] namedParams = _getPublicNostaticFields(this);
  final boolean noIndexParams = _parameters==null;
  final boolean noNamedParams = namedParams.length==0;
  if(noIndexParams&&noNamedParams){return null;}

  final StringBuffer result = new StringBuffer();
  if(!noIndexParams){
    for(int i=0; i<_parameters.length; i++){
      _appendParam(result, Integer.toString(i), _parameters[i]);
    }
  }
  if(!noNamedParams){
    for(int i=0; i<namedParams.length; i++){
      final java.lang.reflect.Field field = namedParams[i];
      final Object fieldValue;
      try{fieldValue= field.get(this);}
      catch(IllegalAccessException ex){continue;}
      _appendParam(result, field.getName(), fieldValue);
    }
  }
  return result.toString();
}

/**Appends to io_buf the external representation "{i_name}=i_value  "
*/
private void _appendParam(
  final StringBuffer io_buf, final String i_name, final Object i_value
){
  io_buf.append("{");   io_buf.append(i_name);   io_buf.append("}=");
  io_buf.append(i_value);   io_buf.append("  ");
}

/**Result: The parameters of this Exc object
*/
public final Object[] parameters(){return _parameters;}

/**Returns the Reflections representation of all public fields in i_object*/
static java.lang.reflect.Field[] _getPublicNostaticFields(final Object i_object){
  final Class theClass = i_object.getClass();
  final String className = theClass.getName();
  final java.lang.reflect.Field[] publicFields = theClass.getFields();
  int statics = 0;
  for(int i=0; i<publicFields.length; i++){ //check no static fields
    final java.lang.reflect.Field field = publicFields[i];
    final int modifiers = field.getModifiers();
    if(java.lang.reflect.Modifier.isStatic(modifiers)){
      publicFields[i] = null;   statics++;
    }
  }
  //Create result with only the nonstatic fields
  final java.lang.reflect.Field[] result
  = new java.lang.reflect.Field[publicFields.length-statics];
  for(int zi=0,qi=0; qi<publicFields.length; qi++){
    final java.lang.reflect.Field field = publicFields[qi];
    if(field!=null){result[zi++] = field;}
  }
  return result;
}

/**Returns all values of the fields i_whichFields in i_fromObject.
  The index sequence for both arrays is the same.
*/
static Object[] _getFieldValues(
  final Object i_fromObject, final java.lang.reflect.Field[] i_whichFields
) throws IllegalAccessException {
  final Object[] result = new Object[i_whichFields.length];
  for(int i=0; i<i_whichFields.length; i++){
    result[i] = i_whichFields[i].get(i_fromObject);
  }
  return result;
}

/** The exception parameters, suitable for message formatting as described
* in http://java.sun.com/docs/books/tutorial/i18n/format/messageFormat.html
* at february 1999.
*/
private final Object[] _parameters;


}//Exc