package lib; //Fio.java

//2000-04-18  Knabe  Name: EndOfFile statt endOfInput
//2000-04-18  Knabe  Ausnahmenamen wie Klassen Gross begonnen
//2000-04-18  Knabe  par(Object) tolerant gegen null-Referenz
//1999-06-03  Knabe  Auch Ausnahme etwasSehrFaul als public
//1998-11-10  Knabe  Als Bestandteil des Pakets 'lib'
//1998-10-17  Knabe  Methode par(name,wert) fuer alle elementaren Typen
//1998-09-25  Knabe  Wichtigstes zuerst, static-Methoden ohne Praefix.
//1998-03-11  Knabe  Umwandlung aus C++
//1993-10-25  Knabe  erstellt

/**Formatted Input/Output. Enthlt Dienste zur Formatierten Ein/Ausgabe
*/
public class Fio {


/**Liest eine Zeile.
  @return deren Interpretation als dezimale int-Zahl >= 0
*/
public static int naechsteZahl()
throws EndOfFile //keine Zeile mehr lesbar wegen Dateiende
     , ZeileLeer  //kein Zeichen in der gelesenen Zeile
     , DataError  //in der gelesenen Zeile wurde ein Zeichen
                  //   angetroffen, welches keine Dezimalziffer ist.
     , ConstraintError //gelesene Zahl auszerhalb des Bereichs von 'int'
     , EtwasSehrFaul //durchgereicht von besonderheitPruefen(char)
     , java.io.IOException //durchgereicht von BufferedReader.readLine()
{
  final int basis = 10;
  int result = 0;
  final String line = _in.readLine();
  if(line==null){throw new EndOfFile();}
  if(line.length()<=0){throw new ZeileLeer();}

  for(int i=0; i<line.length(); i++){ //Zeichenschleife
    final char c = line.charAt(i);
    _besonderheitPruefen(c);
    if(c<'0'||c>'9'){
      //Zeichen, welches keine Dezimalziffer ist
      final int numVal = c;
      throw new DataError(par("i",new Integer(i))
        + par("numVal",new Integer(numVal)) + par("c",new Character(c)));
    }else{ //Dezimalziffer verarbeiten
      final int ziff = Character.digit(c,basis); //Ziffernwert
      int neuzahl = basis*result + ziff;
      if(result != neuzahl/basis){ //Ueberlauf
        throw new ConstraintError();
      }
      result = neuzahl;
    }//if
  }//for
  return result;
}//naechsteZahl

public static class EndOfFile extends Exception{}
public static class ZeileLeer extends Exception{}

public static class DataError extends Exception{
  public DataError(String s) {
    super(s);
  }
}
public static class ConstraintError extends Exception{}

public static class EtwasSehrFaul extends Exception{
  public EtwasSehrFaul(String s) {
    super(s);
  }
}

/** Result: Umwandlung von i_value in lesbare, als i_name benannte Darstellung
  * der Form 'name={wert}  '.
*/
static public String par(final String i_name, final Object i_value)
{
  final String presentation = i_value==null ? "(null)" : i_value.toString();
  return i_name + "={" + presentation + "}  ";
}

/** Result: Umwandlung von i_value in lesbare, als i_name benannte Darstellung
  * der Form 'name={wert}  '.
*/
static public String par(final String i_name, final long i_value)
{
  return par(i_name, Long.toString(i_value));
}

/** Result: Umwandlung von i_value in lesbare, als i_name benannte Darstellung
  * der Form 'name={wert}  '.
*/
static public String par(final String i_name, final double i_value)
{
  return par(i_name, Double.toString(i_value));
}

/** Result: Umwandlung von i_value in lesbare, als i_name benannte Darstellung
  * der Form 'name={wert}  '.
*/
static public String par(final String i_name, final char i_value)
{
  return par(i_name, ""+i_value);
}

/** Result: Umwandlung von i_value in lesbare, als i_name benannte Darstellung
  * der Form 'name={wert}  '.
*/
static public String par(final String i_name, final boolean i_value)
{
  return par(i_name, i_value?"true":"false");
}

/**Zweck:  Diese Prozedur dient nur dazu, eine unbehandelte Ausnahme
        hervorzurufen, um das Folgeverhalten demonstrieren zu koennen.
*/
static private void _besonderheitPruefen(final char i_c)
throws EtwasSehrFaul //i_c ist kein ASCII-Zeichen, d.h. Wert ist >= 128.
{
  if(i_c>=128){
    throw new EtwasSehrFaul(par("i_c", new Character(i_c)));
  }
}

static private java.io.BufferedReader _in
= new java.io.BufferedReader(new java.io.InputStreamReader(System.in));


}//class Fio
