package lib; //Msg.java

import lib.Exc;
import java.lang.String;

//nderungshistorie:
//2000-07-17  Knabe  Auch report(Applet, Throwable)
//2000-07-11  Knabe  Button "Show Stack Trace" verschwindet nach Bettigung
//2000-07-07  Knabe  Meldungstext holen+formatieren verlagert in Klasse MsgText
//2000-07-04  Knabe  report mit Varianten fr Print-Writer/Stream
//2000-06-28  Knabe  Zeilenanzahl der TextArea aus ihrem Inhalt ermitteln
//2000-06-21  Knabe  Auch Anzeige des Stack Trace im Meldungsfenster
//2000-05-23  Knabe  Erweitert um Dialog als owner
//1999-05-20  Knabe  Mit OK-Button zum Quittieren der Meldung per Return/Maus.
//1999-04-29  Knabe  Hintergrundfarbe weiss im Meldungsfenster
//1999-03-23  Knabe  Parametrierung wie in Java-Tutorial/i18n/format
//1999-03-18  Knabe  erstellt

/** Services for reporting messages for exceptions <UL>
    <LI> with parameters (subclasses of Exc)
    <LI> with cause chain and parameters (subclasses of Failed)
    <LI> and standard Java exceptions (other subclasses of java.lang.Throwable)
  </UL>

  @author Christoph Knabe, Berlin, Copyright 1999-2000
*/
public class Msg {


//static message parts:
private static String _className;
private static final String _titlePrefix = "Error Mesage for ";
private static final char _beep = '\u0007';


/**Reports i_throwable blocking io_owner.
  Reports the same contents as report(Frame, Throwable).
  @see #report(java.awt.Frame,Throwable)
  @since JDK 1.2
*/
public static void report(
  final java.awt.Dialog io_owner, final Throwable i_throwable
){
  //comment out the methods marked as "@since JDK 1.2" if compiling with JDK 1.1
  try{new Dialog(io_owner, i_throwable);}
    catch(final Throwable ex){_reportNoWindow(ex,i_throwable);}
  //end try
}
/* */

/**Reports i_throwable from Applet io_owner.
  Reports the same contents as report(Frame, Throwable).
  It seems impossible to block further input to io_owner.
  But nevertheless closing of the browser will close this dialog window, too.
  @see #report(java.awt.Frame,Throwable)
*/
public static void report(
  final java.applet.Applet io_owner, final Throwable i_throwable
){
  try{new Dialog(io_owner, i_throwable);}
    catch(final Throwable ex){_reportNoWindow(ex,i_throwable);}
  //end try
}

/**Reports i_throwable blocking io_owner.
  Firstly reports the same as report(PrintWriter, Throwable)
  into a modal pop-up window of appropriate size.
  Accepting the message is done by <UL>
  <LI> clicking on the "close window" icon, or
  <LI> clicking on the OK-button, or
  <LI> typing <Return> on the OK-button </UL>
  Each of these will close the dialog box.
  <P>
  The Button "Show Stack Trace" will add the compactified
  stack trace including the chain of all causing exceptions
  to the message dialog box. The contents of the compact
  stack trace are described at method
  Failed.printCompactStackTrace(StringBuffer,Throwable,Exception).
  <P>
  Blocks further input to io_owner, until this message
  window will be closed.
  <P>
  In an application this method should not be called directly from
  a GUI class, but the author of each GUI class should set up a method <PRE>
  private void _report(Throwable e){Msg.report(this,e);}
  </PRE>, which will be called from each catch-clause in the event listeners
  and adds the actual GUI class as the owner of the message dialog.

  @see #report(java.io.PrintWriter,Throwable)
  @see Failed#printCompactStackTrace(StringBuffer,Throwable,Exception)
*/
public static void report(
  final java.awt.Frame io_owner, final Throwable i_throwable
){
  try{new Dialog(io_owner, i_throwable);}
    catch(final Throwable ex){_reportNoWindow(ex,i_throwable);}
  //end try
}

/**A message reporting dialog window*/
private static class Dialog {


  /**Creates a dialog window for reporting i_throwable blocking io_owner.
    @see #report(java.awt.Frame,Throwable)
    @since JDK 1.2
  */
  public Dialog (
    final java.awt.Dialog io_owner, final Throwable i_throwable
  ){
    //comment out the methods marked as "@since JDK 1.2" if compiling with JDK 1.1
    _dialog = new java.awt.Dialog(
      io_owner, _titlePrefix + io_owner.getTitle(), true//modal
    );
    _initialize(i_throwable);
  }//Dialog
  /* */

  /**Creates a dialog window for reporting i_throwable from Applet io_owner.
    @see #report(java.awt.Frame,Throwable)
  */
  public Dialog (
    final java.applet.Applet io_owner, final Throwable i_throwable
  ){
    final java.awt.Frame ownerFrame = _getFrame(io_owner);
    _dialog = new java.awt.Dialog(
      _getFrame(io_owner), _titlePrefix + ownerFrame.getTitle(), true//modal
    );
    _initialize(i_throwable);
  }//Dialog

  /**Creates a dialog window for reporting i_throwable blocking io_owner.
    @see #report(java.awt.Frame,Throwable)
  */
  public Dialog (
    final java.awt.Frame io_owner, final Throwable i_throwable
  ){
    _dialog = new java.awt.Dialog(
      io_owner, _titlePrefix + io_owner.getTitle(), true//modal
    );
    _initialize(i_throwable);
  }//Dialog


  /**Returns the frame, to wich i_applet belongs to.
    If the topmost container received by repeated invocation of
    java.awt.Component.getParent() is not a java.awt.Frame, a newly
    created Frame will be returned.
  */
  private static java.awt.Frame _getFrame(final java.applet.Applet i_applet){
    java.awt.Frame result = null;
    for(java.awt.Component c=i_applet; c!=null; c=c.getParent()){
      if(c instanceof java.awt.Frame){result = (java.awt.Frame)c;}
    }
    if(result==null){result = new java.awt.Frame();}
    final String title = result.getTitle();
    if(title==""){result.setTitle("Applet");}
    return result;
  }//_getFrame


  /**Adds to _dialog the necessary components and their listeners*/
  private void _initialize(final Throwable i_throwable){
    _here = new Exception();

    //Look:

    //Button "OK":
    final java.awt.Button okButton
    = new java.awt.Button(" O  K ");
    okButton.setBackground(java.awt.Color.white); //???nur fr FensterFoto
    //Button "Show Stack Trace":
    final java.awt.Button traceButton
    = new java.awt.Button("Show Stack Trace");
    traceButton.setBackground(java.awt.Color.white); //???nur fr FensterFoto
    //buttonPanel:
    /*
    final java.awt.Panel buttonPanel
    = new java.awt.Panel(new java.awt.GridLayout(1,2));
    buttonPanel.add(okButton);
    buttonPanel.add(traceButton);
    */
    final java.awt.Panel buttonPanel
    = new java.awt.Panel(new java.awt.BorderLayout());
    buttonPanel.add(okButton);
    buttonPanel.add(traceButton, java.awt.BorderLayout.EAST);
    _dialog.add(buttonPanel, java.awt.BorderLayout.SOUTH);

    //TextArea for reporting the causer chain messages:
    // empty: 1 row, 80 columns:
    final java.awt.TextArea textArea = new java.awt.TextArea(1,80);
    final String messageChain = MsgText.getMessageChain(i_throwable);
    textArea.setBackground(java.awt.Color.white);
    textArea.setForeground(java.awt.Color.black);
    _append(textArea, messageChain);
    textArea.setEditable(false);
    _dialog.add(textArea, java.awt.BorderLayout.CENTER);

    _dialog.pack();

    //Behaviour:

    okButton.addKeyListener( new java.awt.event.KeyAdapter(){
      public void keyTyped(java.awt.event.KeyEvent ev){
        if(ev.getKeyChar()=='\n'){_dispose();}
      }
    });

    okButton.addActionListener( new java.awt.event.ActionListener(){
      public void actionPerformed(final java.awt.event.ActionEvent i_){
        _dispose();
      }
    });

    traceButton.addKeyListener( new java.awt.event.KeyAdapter(){
      public void keyTyped(java.awt.event.KeyEvent ev){
        if(ev.getKeyChar()=='\n'){
          _fireTraceButton(traceButton, textArea, i_throwable);
        }
      }
    });

    traceButton.addActionListener( new java.awt.event.ActionListener(){
      public void actionPerformed(final java.awt.event.ActionEvent i_){
        _fireTraceButton(traceButton, textArea, i_throwable);
      }
    });

    _dialog.addWindowListener( new java.awt.event.WindowAdapter(){
      public void windowClosing(final java.awt.event.WindowEvent i_){
        _dispose();
      }
    });

    final java.awt.Toolkit toolkit = _dialog.getToolkit();
    toolkit.beep();
    _dialog.setVisible(true);
  }//_initialize()


  /**Free the resources associated with this dialog window*/
  private void _dispose(){_dialog.dispose();}

  /**Appends the stack trace of i_throwable to io_textArea
    and disables io_traceButton
  */
  private void _fireTraceButton(final java.awt.Button io_traceButton,
    final java.awt.TextArea io_textArea, final Throwable i_throwable
  ){
    io_traceButton.setVisible(false);
    final StringBuffer buffer = new StringBuffer();
    buffer.append(MsgText.lineSeparator);   buffer.append(MsgText.lineSeparator);
    buffer.append("----------Stack Trace follows----------");
    buffer.append(MsgText.lineSeparator);
    Failed.printCompactStackTrace(buffer, i_throwable, _here);
    final String stackTrace = buffer.toString();
    _append(io_textArea, stackTrace);
    _dialog.pack();
    _stackTraceShown = true;
  }

  //attributes:

  /**Dialog window for reporting the exception cause chain*/
  private final java.awt.Dialog _dialog;

  /**An Exception encapsulating the stack trace of
    where new Msg.Dialog(...) was called
  */
  private Exception _here;

  private boolean _stackTraceShown = false;

}//Dialog


/**Reports i_throwable to System.err
  @see #report(java.io.PrintWriter,Throwable)
*/
public static void report(final Throwable i_throwable){
  report(System.err, i_throwable);
}

/**Reports i_throwable to io_printer
  @see #report(java.io.PrintWriter,Throwable)
*/
public static void report(
  final java.io.PrintStream io_printer, final Throwable i_throwable
){
  report(new java.io.PrintWriter(io_printer, true), i_throwable);
}

/**Reports i_throwable to io_printer, detail description here.
  Reports the paramerized textual message of i_throwable,
  or if this is not available, the  name and parameters of i_throwable.
  Reports the same for each of the eventually existing chain of causing
  Exception-s.

  The message texts for the exceptions are taken via MsgText.getMessageLine.
  The concept of chaining causing exceptions is described in the class Failed.
  @see MsgText#getMessageLine(Throwable)
  @see Failed
*/
public static void report(
  final java.io.PrintWriter io_printer, final Throwable i_throwable
){
  try{;
    io_printer.print(_beep);
    io_printer.println(MsgText.getMessageChain(i_throwable));
  }
    catch(final Exception ex){
      io_printer.print(_beep);
      io_printer.println(className() + ": COULD NOT BUILD REPORT TEXT FOR:");
      i_throwable.printStackTrace(io_printer);
    }
  //end try
}

/**Prints the compactified stack trace of i_throwable and its
  chained causing exceptions to System.err.
  @see #printStackTrace(java.io.PrintWriter,Throwable)
  */
public static void printStackTrace(final Throwable i_throwable){
  printStackTrace(System.err, i_throwable);
}

/**Prints the compactified stack trace of i_throwable and its
  chained causing exceptions to io_printer.
  @see #printStackTrace(java.io.PrintWriter,Throwable)
  */
public static void printStackTrace(
  final java.io.PrintStream io_printer, final Throwable i_throwable
){
  printStackTrace(new java.io.PrintWriter(io_printer, true), i_throwable);
}

/**Prints the compactified stack trace of i_throwable and its
  chained causing exceptions to io_printer. The contents is provided
  by the method Failed.printCompactStackTrace.
  @see Failed#printCompactStackTrace(StringBuffer,Throwable,Exception)
  */
public static void printStackTrace(
  final java.io.PrintWriter io_printer, final Throwable i_throwable
){
  final StringBuffer buffer = new StringBuffer();
  Failed.printCompactStackTrace(buffer, i_throwable, new Exception());
  io_printer.print(buffer);
  io_printer.flush();
}

/**Returns the name of this class in the format package.class*/
static String className(){
  if(_className==null){_className = new Msg().getClass().getName();}
  return _className;
}

/**Reports to System.err, that exception i_problem occured when
  trying to report the exception i_reportee.
*/
private static void _reportNoWindow(
  final Throwable i_problem, final Throwable i_reportee
){
  System.err.print(className());
  System.err.println(": THE FOLLOWING EXCEPTION OCCURED:");
  i_problem.printStackTrace();
  System.err.print(className());
  System.err.println(": WHEN CONSTRUCTING THE MESSAGE WINDOW FOR EXCEPTION:");
  i_reportee.printStackTrace();
}

/**User should not construct objects of this class*/
private Msg(){}

/**Appends i_text to io_textArea and enlarges the number of rows of this
  TextArea as necessary
*/
private static void _append(
  final java.awt.TextArea io_textArea, final String i_text
){
  io_textArea.setRows( io_textArea.getRows() + _countLines(i_text) );
  io_textArea.append(i_text);
}

/**Result: Number of lines in String i_text
  @see java.io.BufferedReader#readLine()
*/
private static int _countLines(final String i_text){
  final java.io.BufferedReader br
  = new java.io.BufferedReader(new java.io.StringReader(i_text));
  for(int result=0;;result++){
    String line; //final
    try{line = br.readLine();}
      catch(final java.io.IOException ex){line = null;}
    //end try
    if(line==null){return result;}
  }
}


}//Msg


/* Knabe 1999-03-18
Unerklaerlicherweise muss zuerst der traceButton, dann die textArea in den
Dialog eingefuegt werden. Bei der umgekehrten Reihenfolge tritt in der
folgenden Situation:
    nach Ausloesen der Ausname FigurNichtErzeugbar: Rechtspfeil
    und dann Druecken des Buttons <Print Stack Trace>
    und dann Klicken des <Fenster zu>-Kreuzes
die folgende, unmotivierte Ausnahme auf:
    Exception occurred during event dispatching:
    java.lang.NullPointerException: invalid peer
        at java.awt.Component.removeNotify(Component.java:2523)
        at java.awt.TextComponent.removeNotify(TextComponent.java:106)
        at java.awt.Container.removeNotify(Container.java:1147)
        at java.awt.Window.dispose(Window.java:177)
        at lib.Msg$1.windowClosing(Msg.java:63)
*/